import starrail.utils.babelfish.dictionary as dictionary
from starrail.utils.babelfish.translate import translate

from . import constants

average_gacha_per_5_star = dictionary.average_gacha_per_5_star
gacha_report = dictionary.gacha_report
gacha_title = dictionary.gacha_title
history_of_5_stars = dictionary.history_of_5_stars
html_thead = dictionary.html_thead
markdown_thead = dictionary.markdown_thead
ui_about = dictionary.ui_about
ui_about_this = dictionary.ui_about_this
ui_auto = dictionary.ui_auto
ui_cancel_update = dictionary.ui_cancel_update
ui_check_update = dictionary.ui_check_update
ui_check_update_at_start = dictionary.ui_check_update_at_start
ui_check_update_at_start_desc = dictionary.ui_check_update_at_start_desc
ui_check_update_fail = dictionary.ui_check_update_fail
ui_check_update_success = dictionary.ui_check_update_success
ui_copyright = dictionary.ui_copyright
ui_curr_fps = dictionary.ui_curr_fps
ui_current_version = dictionary.ui_current_version
ui_deduce_uid_fail = dictionary.ui_deduce_uid_fail
ui_downloading_gacha = dictionary.ui_downloading_gacha
ui_en = dictionary.ui_en
ui_extract_api_fail = dictionary.ui_extract_api_fail
ui_extract_api_fail_with_msg = dictionary.ui_extract_api_fail_with_msg
ui_extracting_api_url = dictionary.ui_extracting_api_url
ui_fine = dictionary.ui_fine
ui_fine1 = dictionary.ui_fine1
ui_fine2 = dictionary.ui_fine2
ui_fps_is_already = dictionary.ui_fps_is_already
ui_fps_is_set_to = dictionary.ui_fps_is_set_to
ui_gacha_basic_prob = dictionary.ui_gacha_basic_prob
ui_gacha_count = dictionary.ui_gacha_count
ui_gacha_since_last = dictionary.ui_gacha_since_last
ui_gacha_sync = dictionary.ui_gacha_sync
ui_gacha_sync_desc = dictionary.ui_gacha_sync_desc
ui_gacha_true_prob = dictionary.ui_gacha_true_prob
ui_gacha_type = dictionary.ui_gacha_type
ui_get_started = dictionary.ui_get_started
ui_get_started_desc = dictionary.ui_get_started_desc
ui_github_repo = dictionary.ui_github_repo
ui_github_repo_desc = dictionary.ui_github_repo_desc
ui_is_latest_version = dictionary.ui_is_latest_version
ui_locale = dictionary.ui_locale
ui_locale_setting_desc = dictionary.ui_locale_setting_desc
ui_no_data = dictionary.ui_no_data
ui_not_good = dictionary.ui_not_good
ui_ooops = dictionary.ui_ooops
ui_open_docs = dictionary.ui_open_docs
ui_open_issues = dictionary.ui_open_issues
ui_open_repo = dictionary.ui_open_repo
ui_open_troubleshooting = dictionary.ui_open_troubleshooting
ui_operation_zone = dictionary.ui_operation_zone
ui_personalization = dictionary.ui_personalization
ui_reset_fps = dictionary.ui_reset_fps
ui_reset_panel = dictionary.ui_reset_panel
ui_save_data = dictionary.ui_save_data
ui_save_success = dictionary.ui_save_success
ui_save_success_msg = dictionary.ui_save_success_msg
ui_send_feedback = dictionary.ui_send_feedback
ui_send_feedback_desc = dictionary.ui_send_feedback_desc
ui_set_fps_fail_with_msg = dictionary.ui_set_fps_fail_with_msg
ui_setting_failure = dictionary.ui_setting_failure
ui_setting_panel = dictionary.ui_setting_panel
ui_setting_restart_content = dictionary.ui_setting_restart_content
ui_setting_restart_title = dictionary.ui_setting_restart_title
ui_setting_success = dictionary.ui_setting_success
ui_settings = dictionary.ui_settings
ui_status_panel = dictionary.ui_status_panel
ui_sync = dictionary.ui_sync
ui_sync_gacha_fail = dictionary.ui_sync_gacha_fail
ui_sync_gacha_initial = dictionary.ui_sync_gacha_initial
ui_sync_gacha_success = dictionary.ui_sync_gacha_success
ui_synchronizing_gacha = dictionary.ui_synchronizing_gacha
ui_theme_mode = dictionary.ui_theme_mode
ui_theme_mode_dark = dictionary.ui_theme_mode_dark
ui_theme_mode_light = dictionary.ui_theme_mode_light
ui_theme_mode_settings_desc = dictionary.ui_theme_mode_settings_desc
ui_title = dictionary.ui_title
ui_traceback = dictionary.ui_traceback
ui_troubleshooting = dictionary.ui_troubleshooting
ui_troubleshooting_desc = dictionary.ui_troubleshooting_desc
ui_unlock_fps = dictionary.ui_unlock_fps
ui_unlock_fps_desc = dictionary.ui_unlock_fps_desc
ui_update_available = dictionary.ui_update_available
ui_update_desc = dictionary.ui_update_desc
ui_users = dictionary.ui_users
ui_users_desc = dictionary.ui_users_desc
ui_welcome = dictionary.ui_welcome
ui_zhs = dictionary.ui_zhs

__all__ = ['constants', 'translate']
