REPO_URL = 'https://github.com/LittleNyima/honkai-starrail-toolkit'
DOCS_URL = f'{REPO_URL}/tree/develop/docs'
TROUBLESHOOTING_URL = f'{DOCS_URL}/troubleshooting.md'
ISSUE_URL = f'{REPO_URL}/issues'
TIME_FMT = '%Y-%m-%d_%H:%M:%S'
